import logging

from pathlib import Path
from xmlschema import *

from .TypeMapping import *
from .XSDNativeSimpleTypeMapping import XSDNativeSimpleTypeMapping
from .ChoiceMapping import ChoiceMapping
from .SequenceMapping import SequenceMapping


log = logging.getLogger()

mappings = [
    XSDNativeSimpleTypeMapping(),
    ChoiceMapping(),
    SequenceMapping(),
]

def translate_schema (schema_path: str, ns_to_id_map: dict, schema_locations = []):
    js = {
        "$id" : "?",
        "$defs" : {}
    }

    logging.info(f"Translating schema {schema_path}")
    xs = XMLSchema(schema_path, validation='lax', locations=schema_locations)
    logging.info(f"Schema namespace: {xs.target_namespace}" )

    schema_id = ns_to_id_map[xs.target_namespace]["id"]
    js['$id'] = schema_id

    TypeMapping.ns_to_id_map = ns_to_id_map

    elementList = []
    for elementName, element in xs.elements.items():
        logging.info(f"Processing element {elementName} : {element}")
        elementList.append(TypeMapping.get_ref_for(element.type, element.namespaces['']))
    if len(elementList) == 1:
        js['$ref'] = elementList[0]['$ref']
    elif len(elementList) > 1:
        js['oneOf'] = elementList

    descendent_types = {}
    for type_name, xsd_type in xs.types.items():
        logging.info(f"Processing {type_name} : {xsd_type}")

        j = None
        for mapping in mappings:
            log.debug("\n----------------------------------------")
            j = mapping.map(xsd_type)
            if j is None:
                continue
            else:
                break
        if j is None:
            raise Exception(f"Unmapped type {type_name} ({xsd_type})")
        js["$defs"][xsd_type.local_name] = j
        logging.debug (f"Mapped {type_name} to {j}")

    print (descendent_types)
    return js


